# eslint-plugin-react-component-name

## Motivation

- The default ESLint rules for React checking `displayName` property of a component, but has no autofix.
- There is a few bundlers which support automatic `displayName` inlining, but it isn't universal.
- We want to support all kind of decorators like `observer` from 'mobx-react' or `reatomComponent` from '@reatom/npm-react'.
- It is important to use **named function** instead of additional `displayName`, to avoid ambiguity in error **stack traces**.

---

## Installation

Install the package using npm or yarn:

```bash
npm install eslint-plugin-react-component-name --save-dev
```

## Usage

Add the plugin to your `.eslintrc` configuration:

```json
{
  "plugins": ["react-component-name"],
  "rules": {
    "react-component-name/react-component-name": [
      "error",
      {
        "targets": ["memo", "forwardRef", "observer"]
      }
    ]
  }
}
```

- **`targets`**: An array of React higher-order function names where named functions should be enforced. Defaults to `["memo"]`.

---

## Examples

### Wrong Code

```javascript
const MyComponent = memo(() => {
  return <div>Hello</div>;
});

const MyRef = forwardRef((props, ref) => {
  return <input ref={ref} {...props} />;
});
```

### Fixed Code

```javascript
const MyComponent = memo(function MyComponent() {
  return <div>Hello</div>;
});

const MyRef = forwardRef(function MyRef(props, ref) {
  return <input ref={ref} {...props} />;
});
```

---

## Rule Details

This rule ensures that functions used with React helpers like `memo`, `forwardRef`, or `observer` are explicitly named. Unnamed (anonymous) functions are flagged, and ESLint can provide auto-fix suggestions by converting them into named functions that match the variable they are assigned to.

### Why Naming Matters

1. **Improved Stack Traces**: Named functions provide meaningful names in error stack traces, simplifying debugging.
2. **Code Clarity**: Explicit naming enhances code readability, making it easier for developers to understand the intent.

---

## Motivation for ESLint-Based Enforcement

Using ESLint for this task is more efficient and consistent than bundler-based solutions:

- **Ecosystem Integration**: ESLint is already widely used for code linting, ensuring you don't need to rely on additional plugins or tools.
- **Static Analysis**: ESLint enforces rules at the source code level, making it easier to detect and fix issues before they reach runtime.
- **Tool Consolidation**: Avoids adding unnecessary complexity to your build pipeline with bundler plugins.

This plugin seamlessly integrates into your existing ESLint setup, enforcing best practices without additional overhead.
